require "utils"

-- Memoizers (not a typo) -----------------------------------------------
local imgdir = "img/"
imgs = {}
local imgs_mt = {__index = function(s, k)
	if type(k) ~= 'string' then return nil end
	local I
	if love.filesystem.exists(imgdir..k..".png") then
		I = love.graphics.newImage(imgdir..k..".png")
	elseif love.filesystem.exists(imgdir..k..".jpg") then
		I = love.graphics.newImage(imgdir..k..".jpg")
	else
		error("could not load image 'img/"..tostring(k)..".(png/jpg)'!")
	end
	rawset(s, k, I)
	return I
end
}setmetatable(imgs, imgs_mt)

fonts = {}
local fonts_mt = {__index = function(s, k)
	if type(k) ~= 'number' then return nil end
	local f = love.graphics.newFont(k)
	rawset(s, k, f)
	return f
end
}setmetatable(fonts, fonts_mt)

-------------------------------------------------------------------------

local card_size = 100
local font = 10
local bg_col = color(.51,.75,.94);

function drawcard(card, x, y, s)
	assert(card and (x and y), "must be given a drawable card, and coords")
	assert(type(card) == 'table', "infobox: parm#1 expected table, got "..type(info))
	assert(type(x)    == 'number', "infobox: parm#2 expected number, got "..type(x))
	assert(type(y)    == 'number', "infobox: parm#3 expected number, got "..type(y))
	s = s or 1
	
	local lg = love.graphics
	local col = card.color or bg_col
	local img = imgs[(card.active and card.image) or 'generic']
	lg.setColor(col)
	lg.rectangle('fill', x, y, card_size*s, card_size*s)
	lg.setColor(255, 255, 255)
	lg.draw(img, x, y, 0, card_size/img:getWidth()*s)
	lg.rectangle('line', x+(1*s)+.5, y+(1*s)+.5, (card_size-2)*s, (card_size-2)*s)
	
	if card.active then
		local f = lg.getFont()
		lg.setFont(fonts[font*s])
		lg.print(card.name, x+(3*s), y+((card_size-3)*s), -math.rad(90))
	end
end

function infobox(info, x, y, s)
	assert(info and (x and y), "Must be given a drawable card, and coords")
	assert(type(info) == 'table', "infobox: parm#1 expected table, got "..type(info))
	assert(type(x) == 'number', "infobox: parm#2 expected number, got "..type(x))
	assert(type(y) == 'number', "infobox: parm#3 expected number, got "..type(y))
	assert((not s) or type(s) == 'number', "infobox: parm#4 expected number, got "..type(s))
	if info.active == false then return end
	assert(info.name and info.text, "Info must have a name, and text, or be inactive")
	s = s or 1
	local w = info.width or 320

	local f = fonts[font*s]
	local w, l = f:getWrap(info.text, w)
	l = l+2
	w = w*1.3
	local h = info.height or ((l+1.5) * f:getHeight()) + ((fonts[font*s]:getHeight()+3)*1.2)
	
	local lg = love.graphics

	do
		local sw, sh = lg.getWidth(), lg.getHeight()
		if x+w >= sw then x = sw-w end
		if y+h >= sh then y = sh-h end
	end
	
	local col = info.color or bg_col
	lg.setColor(col)
	lg.rectangle('fill', x, y, w*s, h*s)
	
	lg.setColorMode('modulate')
	if grey(col) < .5 then
		lg.setColor(255, 255, 255)
	else
		lg.setColor(0, 0, 0)
	end
	lg.rectangle('line', x+(1*s)+.5, y+(1*s)+.5, (w-2)*s, (h-2)*s)
	
	local f = lg.getFont()
	lg.setFont(fonts[(font+4)*s])
	lg.print(info.name, x+(3*s), y+(3*s))
	local off = lg.getFont():getHeight()*1.2
	lg.setFont(fonts[font*s])
	lg.printf(info.text, x+(6*s), y+off+(3*s), (w-12)*s)
--	lg.setScissor(unpack(sis))
	lg.setColorMode('replace')

end

local function plrstr(plr)
	local resources = ((plr.ammo + plr.supplies + #plr.deck)/3)
	resources = resources < 25 and resources or 25
	resources = resources / 25
	local health = plr.health -- - (plr.plague*2)
	health = health / plr.healthmax
	local strength = resources * health
	return strength
end



function ai_decide(self)
	assert(type(self) == 'table')
	
	local opp = players[(plr == players[1] and 2) or 1]
	assert(opp ~= plr, "Could not determine opponent")

	local relative_strength = plrstr(self) - plrstr(opp)
	
	print(self.name.."'s relative strength:", relative_strength, relative_strength > 0  and "^_^" or ";_;")

	local couldplay, next = false
	for i, c in pairs(self.hand) do
		next = false
		if not self:canafford(c) then next = true end
		if (c.hints.sacrifice and relative_strength < 0) then next = true end
		if (c.hints.mutual and relative_strength < 0) then next = true end
		if (c.hints.cure and self.plague <= 0) then next = true end
		if (c.hints.heal and (self.health/self.healthmax) >= .75) then next = true end
		print("ai_health percent", (self.health/self.healthmax))
		if not next then
			self:play(i)
			couldplay = true
		end
	end
	if not couldplay then
		if #self.hand >= 7 then
			local c, i = pick(self.hand)
			self:discard(i)
		end
	end
		
end


