local _M = {_NAME = "deck", _TYPE = 'module'}

require "utils"
-------------------------------------------------------------------------

_M.suit_action     = color(.0,.6,.0);
_M.suit_attack     = color(.6,.0,.0);
_M.suit_summon     = color(.6,.0,.6);
_M.suit_general    = color(.6,.6,.6);
_M.suit_heal       = color(.0,.0,.6);
_M.suit_loot       = color(.9,.8,.2);
_M.suit_misc       = color(.51,.75,.94);

_M.suit_military   = color(.4,.5,.1);
_M.suit_police     = color(.1,.1,.7);
_M.suit_institute  = color(.5,.2,.7);
_M.suit_logistics  = color(.8,.1,.1);
_M.suit_survivors  = color(.6,.4,.1);
_M.suit_core       = color(.6,.6,.6);

function _M:newcard(t)
	local c = {}
	
	c._TYPE  = 'card'
	c.name   = t.name or "Card"
	c.type   = t.type or 'general'
	c.active = t.active or true
	c.text   = t.text or ""
	c.image  = t.image or 'general'
	c.color  = t.color or _M.suit_general
	c.effect = t.effect or {}
	c.cost   = t.cost or {}
	c.hints  = t.hints or {}
	c.set  = t.set or {}
	local tmp = {}
	for k, v in pairs(c.hints) do
		tmp[v]=true
	end
	c.hints = tmp
	
	return c
end

function _M:clonedeck(t)
	local d = {}
		table.foreach(t, function(k, v) d[k] = _M:newcard(v) end)
	return d
end

local set = {}
_M.set = set

local count = {}

local function addcard(c)
	c = _M:newcard(c)
	if not set[c.name] then
		set[c.name] = c
		set[#set+1] = c
		count[c.set[1]] = (count[c.set[1]] or 0) +1
	end
end

function _M:load(fname)
	local stack = {}
	local lf = love.filesystem;
	if lf.exists(fname) then
		-- l.filesystem has 'more than a few' limitations, so we'll just use io.* since its one weakness doesn't apply here.
		local savdir = lf.getSaveDirectory()
		local file, msg = io.open(string.format("%s/%s", savdir, fname), 'r')
		if file then
			local line, size
			line = file:read'*l'
			size = file:read'*l'
			if line == 'Deck 1.0' and size == size:match("size: 25") then
				local total, line = 0, file:read'*l'
				while total < 25 and line do
					if line ~= "" then
						local count, name = line:match("(%d+)x%s+(.*)")
						if count then for i=1,count do
							stack[#stack+1] = deck:newcard(deck.set[name]) end
						end
						total = total+count
					end
					line = file:read'*l'
				end
			end
			file:close()
		else error(msg or "no message :/") end
	end

	return stack
end

addcard{name = "Bandages", type='action';
	text = "Supplies-3, health+5\nA tight wrap of gauze can be surprisingly effective.";
	set = {'C'};
	cost = {supplies=3};
	hints = {'heal'};
	image = 'heal';
	color = _M.suit_core;
	effect = {{'supplies', -3},{'health', 5}};
}

addcard{name = "Vaccine", type='action';
	text = "Supplies-10, plague set to 0\nNo one knows where the vaccine came from, but it works."; 
	set = {'I'};
	cost = {supplies=10,plague=1};
	hints = {'cure'};
	image = 'heal';
	color = _M.suit_institute;
	effect = {{'plague', -1e1000},{'supplies', -10}};
}
	
addcard{name = "Plagueflies", type='summon';
	text = "All players plague+2\nPlague-ridden flies grow fat upon the countless corpses.";
	set = {'C'};
	cost = {};
	hints = {'debuff', 'mutual'};
	image = 'summon';
	color = _M.suit_core;
	effect = {{'plague', 2}, {'opponent',{'plague', 2}}};
}
	
addcard{name = "Grave Goods", type='action'; 
	text = "Supplies+3, plague+1\nLooting plague ridden corpses can be profitable, but risks disease.";
	set = {'C'};
	image = 'loot';
	hints = {'sacrifice', 'supplies'};
	cost = {};
	color = _M.suit_core;
	effect = {{'supplies', 3}, {'plague', 1}};
}
	
addcard{name = "Iodine", type='action'; 
	text = "Plague-2, health-3\nThe sting of Iodine on an open wound is not soon forgotten, but it kills the plague.";
	set = {'C'};
	cost = {health=3, plague=1};
	hints = {'sacrifice', 'cure'};
	image = 'heal';
	color = _M.suit_core;
	effect = {{'health', -3}, {'plague', -2}};
}

addcard{name = "Bloodlust", type='action'; 
	text = "Ammo-5, Heal 3 Plague and Health\nSometimes it helps to let off some steam.";
	set = {'S'};
	cost = {ammo=5};
	hints = {'sacrifice', 'cure'};
	image = 'heal';
	color = _M.suit_survivors;
	effect = {{'health', 3}, {'plague', -3}, {'ammo', -5}};
}

addcard{name = "Full-Auto", type='action'; 
	text = "ammo-10, deals 15 damage\nSometimes the best strategy is just to hold down the trigger until they stop moving...";
	set = {'M'};
	cost = {ammo=10};
	hints = {'attack'};
	image = 'attack';
	color = _M.suit_military;
	effect = {{'opponent', {'health', -15}}, {'ammo', -10}};
}

addcard{name = "Six-shot", type='action'; 
	text = "Ammo-5, deals 7 damage\nSix-shots can be reliable if you know how to treat them.";
	set = {'C'};
	cost = {ammo=5};
	hints = {'attack'};
	image = 'attack';
	color = _M.suit_core;
	effect = {{'opponent', {'health', -7}}, {'ammo', -5}};
}

addcard{name = "Demo Charge", type='action'; 
	text = "Ammo-5, deals 10 damage to both players\nNobody escapes the bomb...";
	set = {'M'};
	cost = {ammo=5, health=10};
	hints = {'attack', 'mutual'};
	image = 'attack';
	color = _M.suit_military;
	effect = {{'opponent', {'health', -10}}, {'ammo', -5}, {'health', -10}};
}

addcard{name = "Grenade", type='action'; 
	text = "Ammo-3, deals 7 damage to both players\nGrenading in close quarters is an act of desperation.";
	set = {'S'};
	cost = {ammo=3, health=7};
	hints = {'attack', 'mutual', 'sacrifice'};
	image = 'attack';
	color = _M.suit_survivors;
	effect = {{'opponent', {'health', -7}}, {'ammo', -3}, {'health', -7}};
}

addcard{name = "Craft Shells", type='action'; 
	text = "Supplies-5, ammo+3\nHome-made ammo and explosives are common after the plague.";
	set = {'S'};
	cost = {supplies=5};
	hints = {'sacrifice', 'loot'};
	image = 'loot';
	color = _M.suit_survivors;
	effect = {{'supplies', -5}, {'ammo', 3}};
}

addcard{name = "Salvage", type='action'; 
	text = "Supplies+3, Ammo-5\nTearing apart ammunition is rarely productive, but occasionally appropriate ";
	set = {'S'};
	cost = {ammo=5};
	hints = {'sacrifice', 'loot'};
	image = 'loot';
	color = _M.suit_survivors;
	effect = {{'supplies', 3}, {'ammo', -5}};
}

addcard{name = "Scavenge", type='action'; 
	text = "Supplies+5\nScavenging for supplies is vital pastime.";
	set = {'C'};
	cost = {};
	hints = {'loot'};
	image = 'loot';
	color = _M.suit_core;
	effect = {{'supplies', 5}};
}

addcard{name = "Secret Cache", type='action'; 
	text = "Supplies+3, ammo+3\nSurviors often secret away small stashes of vital supplies.";
	set = {'S'};
	cost = {};
	hints = {'loot'};
	image = 'loot';
	color = _M.suit_survivors;
	effect = {{'supplies', 3},{'ammo', 3}};
}

addcard{name = "Weapons Stockpile", type='action'; 
	text = "Ammo+7\nThe wise keep weapons and ammo somewhere safe, just in case.";
	set = {'M'};
	cost = {};
	hints = {'loot'};
	image = 'loot';
	color = _M.suit_military;
	effect = {{'ammo', 7}};
}

addcard{name = "Raid", type='action'; 
	text = "Supplies-3, Ammo+3, opponent looses 5 Ammo\nConducting raids is a good way to make enemies.";
	set = {'MI'};
	cost = {supplies=3};
	hints = {'loot'};
	image = 'action';
	color = _M.suit_police;
	effect = {{'supplies', -3}, {'ammo', 3}, {'opponent',{'ammo', -5}}};
}

addcard{name = "Diversion", type='action'; 
	text = "Ammo-3, your opponent is distracted.\nWell-timed Diversions can put people in unfortunate positions";
	set = {'I'};
	cost = {ammo=3};
	hints = {'debuf'};
	image = 'action';
	color = _M.suit_institute;
	effect = {{'opponent',{'distract', 1}},{'supplies', -3}};
}

addcard{name = "Manufacture", type='action'; 
	text = "Supplies+7, You do not gain a card next turn\nIf one is willing to spend the time, they can make the most useful things.";
	set = {'I'};
	cost = {};
	hints = {'sacrifice'};
	image = 'loot';
	color = _M.suit_institute;
	effect = {{'distract', 1}, {'supplies', 7}};
}

addcard{name = "Rush Job", type='action'; 
	text = "Draw 2, you don't draw next turn\nSometimes on-time is too late.";
	set = {'SI'};
	cost = {};
	hints = {'sacrifice'};
	image = 'action';
	color = _M.suit_logistics;
	effect = {{'distract', 1}, {'draw', 2}};
}

addcard{name = "Requisition", type='action'; 
	text = "Supplies+5, Ammo+3\nSigned in triplicate, overnight shipping.";
	set = {'I'};
	cost = {};
	hints = {};
	image = 'loot';
	color = _M.suit_institute;
	effect = {{'shipping', 1, {{'ammo', 3},{'supplies', 5}};}};
}

addcard{name = "Drug Cocktail", type='action'; 
	text = "Plague-5, health-7\nSome packages contained interesting substances.";
	set = {'SI'};
	cost = {health=7,plague=1};
	hints = {'cure', 'sacrifice'};
	image = 'heal';
	color = _M.suit_logistics;
	effect = {{'health', -7}, {'plague', -5}};
}

addcard{name = "Diverted Package", type='action'; 
	text = "supplies-3, Draw 1 card, your opponent does not draw a card next turn.\nNot all packages reached their destination.";
	set = {'SI'};
	cost = {supplies=3};
	hints = {'resource'};
	image = 'loot';
	color = _M.suit_logistics;
	effect = {{'supplies',-3}, {'opponent',{'distract', 1}}, {'draw', 1}};
}

print(string.format("Deck contains %d cards, divided as follows:", #set))
tdump(count)
print("[end]")

-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M



