local _M = {_NAME = "effect", _TYPE = 'module'}
-------------------------------------------------------------------------
local ef_mt = {__index=_M}

local cnt = 0

local icon_generic = {'gray00', 'gray01', 'gray02', 'gray03',  'gray04', 'gray05', 'gray06'}
function _M:new(t, prm)
	assert(t)
	local e = {}
	setmetatable(e, ef_mt)
	cnt=cnt+1
	
	e.name = t.name or 'effect#'..cnt
	e.type = t.type or tostring(e.name):lower()
	e.text = t.text or ""
	e.color = t.color or {80,80,80}
	e.onturn = t.onturn
	e.onend = t.onend
	e.onapply = t.onapply
	e.number = t.number or 0
	e.duration = t.duration or e.number
	e.icon = t.icon or icon_generic
	e.merge = t.merge
	e.parms = prm
	
	return e
end

function _M:onapply() end
function _M:onturn() end
function _M:onapply() end

function _M:maybedie(s)
	if self.number <= 0 then
		s:removeEffect(self)
	end
end

local function simplemerge(s, e)
	assert(e.type == s.type, "Can't merge disparate effects!")
	s.number = s.number + e.number;
	return s
end;


local set = {}
_M.set = set

local function addeffect(e)
	e = _M:new(e)
	if not set[e.name] then
		set[e.type] = e
		set[#set+1] = e
	end
end

addeffect{name="Plague",type='plague';
	text = "The virus is an ever-present threat to your health, which degrades each turn you are infected.";
	number = 1;
	icon = {'green00', 'green01', 'green02', 'green03',  'green04', 'green05', 'green06'};
	onturn = function(s, sub)
		sub.health = sub.health - s.number
		local msg = string.format("The plague takes its toll on %s.", sub.name)
		addlog('event', msg ,{name=s.name..": "..s.duration, text=s.text, color=s.color})
		s:maybedie(sub)
	end;
	merge = simplemerge;
}

addeffect{name="Distracted",type='distract';
	text = "Distraction can be costly, You gain no cards at the begining of your turn.";
	number = 1;
	onturn = function(s, sub)
		local msg = string.format("%s's distraction as cost them a card.", sub.name)
		addlog('event', msg ,{name=s.name..": "..s.duration, text=s.text, color=s.color})
		s.number = s.number-1
		s:maybedie(sub)
	end;
	merge = simplemerge;
}

addeffect{name="Irradiated",type='irradiate';
	text = "Irradiation slowly cures the plague, but you may not heal.";
	number = 1;
	onturn = function(s, sub)
		local msg = string.format("%s's glow cures some of the plague.", sub.name)
		addlog('event', msg ,{name=s.name..": "..s.duration, text=s.text, color=s.color})
		s.number = s.number-1
		s:maybedie(sub)
	end;
	merge = simplemerge;
}

addeffect{name="Shipping",type='shipping';
	text = "Some packages take time to arrive.";
	number = 1;
	onturn = function(s, sub)
		s.number = s.number-1
		if s.number <=0 then
			local msg = string.format("%s's package has arrived.", sub.name)
			addlog('event', msg ,{name=s.name..": "..s.duration, text=s.text, color=s.color})

			sub:applyEffects(s.parms)
			
			sub:removeEffect(s)
		end
	end;
	merge = simplemerge;
}

-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M

