local _M = {_NAME = "ggdeckedit", _TYPE = 'module'}

local panel = require "panel"
local adds = require "adds"
local button = require "panel.button"
local deck = require "deck"
-------------------------------------------------------------------------

setmetatable(_M, {__index=panel})
local newsaveslot;
local newcardslot;

function _M:new(t)
	t=t or {}
	local dedit = panel:new{x=ui.width*.1, y=ui.height*.1, width=ui.width*.8, height=ui.height*.8}
	dedit.name = "Deck Editor"
	dedit.valign = 'bottom'
	dedit.font = 32
	dedit.deck = {}
	setmetatable(dedit, {__index=_M})


	local decked = dedit.deck
	

	local function onsave(self)
		local lf = love.filesystem;
		if not lf.exists("decks/") then lf.mkdir("decks") end

		local file, msg = lf.newFile(string.format("decks/deck%02d.deck", self.slot))
		fate = file:open'w'
		if fate then
			file:write(string.format("Deck 1.0\nsize: %d\n\n", #dedit.deck))
			local condense = {}
			for i=1, (#dedit.deck <= 25 and #dedit.deck) or 25 do
				condense[dedit.deck[i].name] = (condense[dedit.deck[i].name] or 0) + 1
			end
			for name, count in pairs(condense) do
				file:write(string.format("%dx %s\n", count, name))
			end
			file:write'\n'
			file:close()
		else
			error(msg or "no message :/")
		end
	end

	local function onload(self)
		table.foreach(dedit.deck, function(k, v) dedit.deck[k] = nil end) -- clear the deck

		local d = deck:load(string.format("decks/deck%02d.deck", self.slot))
		for i=1,#d do dedit.deck[i] = d[i] end
	end

	for i=1,8 do
		local sslot = newsaveslot(dedit.width-154,(i*2)+4+(i-1)*50)
		sslot.slot = i
		sslot.onsave = onsave
		sslot.onload = onload
		dedit:add(sslot)
	end

	for i=1, 25 do
		decked[#decked+1] = deck:newcard(pick(deck.set))
	end

	for y=1,5 do
		for x=1,5 do
			local idx = x+((y-1)*5)
			local card = newcardslot((x-1)*75, (y-1)*75, decked, idx)
			function card:onclick(x, y, btn)
				local dim = 75*5
				local idx = self.slot
				local decker = self.deck
				local picker = newdeckpicker{x=(ui.width-dim)/2, y=(ui.height-dim)/2, width=dim, height=dim}
				function picker:result(r)
					assert(r, "What the...?")
					decker[idx] = r
				end
				ui:addModal(picker)
			end
			dedit:add(card)
		end
	end

	local done = button:new{name="done", text="Done", x=dedit.width-154, y=dedit.height-54, width=150, height=50}
	function done:onclick()
		self.parent:close()
		self.parent:result()
	end
	dedit:add(done)


	return dedit	
end	

function _M:result()
	self:onresult(self.deck)
end

function newcardslot(x, y, deck, idx)
	local slot = button:new{x=x, y=y, width=75, height=75}
	assert( deck and idx, "Must be given a valid deck and slot index")
	slot.deck = deck
	slot.slot = idx
	
	function slot:ondraw()
		local card = self.deck[self.slot] or {}
		drawcard(card, 0, 0, .75)
	end

	function slot:onupdate()
		self.info = self.deck[self.slot] or {}
	end
	
	return slot
end

function newdeckpicker(t)
	t = t or {}
	picker = panel:new(t)
	local decked = deck.set
	picker.deck = decked
	
	function picker:result(r)
		self:onresult(r)
	end

	local function s_result(self, r) self.parent:result(r) end	
	local function s_onclose(self) self.parent:close() end	

	for y=1,5 do
		for x=1,5 do
			local idx = x+((y-1)*5)
			if idx <= #decked then
				local card = newcardslot((x-1)*75, (y-1)*75, decked, idx)
				card.deck = decked
				card.result = s_result
				card.onclose = s_onclose
				function card:onclick(x, y, btn)
					self:result(self.deck[self.slot])
					self:close()
				end
				picker:add(card)
			end
		end
	end

	return picker
end

function newsaveslot(x, y)
	local slot = panel:new{x=x, y=y, width=150, height=50}
	slot.name = "Save Slot#".. slot.id
	slot.text = ""
	slot.onsave = function()end
	slot.onload = function()end
	
	local caption = panel:new{width=slot.width-50, height=slot.height}
	caption.text = "Save Slot"
	slot:add(caption)

	local load = button:new{x=slot.width-50, y=0, width=50, height=slot.height/2}
	load.name = "Load"
	load.color_bg = color(.2,.8,.2)
	load.onclick = function(self, x, y, btn)
		self.parent:onload()
	end
	slot:add(load)
	local save = button:new{x=slot.width-50, y=load.height, width=50, height=slot.height/2}
	save.name = "Save"
	save.color_bg = color(.8,.2,.2)
	save.onclick = function(self, x, y, btn)
		self.parent:onsave()
	end
	slot:add(save)

	return slot
end

-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M

