local _M = {_NAME = "gggame", _TYPE = 'module'}

local panel = require "panel"
local button = require "panel.button"
local player = require "player"
local ggfinale = require "ggfinale"
-------------------------------------------------------------------------

local panel_mt ={__index = panel}
local game_mt ={__index = _M}
setmetatable(_M, panel_mt)

function _M:new(t)
	t = t or {}
	local g = panel:new(t)
	setmetatable(g, game_mt)
	g.color_bg = {0,0,0,0}
	g.text = ""
	
	players = {}
	players[1] = player:new{name = "Survivor", deck = t.playerdeck}
	players[2] = player:new{name = "Looter", hostile = true}

	turn = button:new{name="endturn", text="End Turn", x=800-100, y=600-100, width=100, height=50;
	color_bg = color(.0,.6,.3), color_fg = color(.0,.2,.1)}
	turn.info = {name = "End Turn", text = "Ends your current turn, and begins the next one."}
	function turn:onclick(x, y, btn)
		print(self.name, self._TYPE)
		self.parent:doturn()
		addlog('system', "Turn advanced")
	end
	g:add(turn)
	
	loggerinit(g)


	local function deckupdate(self)
		self.name = #self.player.deck
	end

	local function newhand(n)
		local hnd = {}
		for i=1,n do
			hnd[#hnd+1] = deck:newcard(pick(deck.set))
		end
		return hnd
	end

	hand_player   = player:newhand{name="Player's hand",   size=7, player = players[1], position = 3}
	g:add(hand_player)
	hand_opponent = player:newhand{name="Opponent's hand", size=7, player = players[2], position = 1}
	g:add(hand_opponent)

	tmp = ggplayerstatus:new{x=0, y=450, player=players[1]}
	g:add(tmp)
	tmp = ggplayerstatus:new{x=0, y=100, player=players[2]}
	g:add(tmp)

	return g
end

function loggerinit(g)
	log = {}
	local log_mt = {}
	function log_mt.__index(l, k)
		if type(k) == 'number' and k < 0 then
			k = -(k+1)
			return rawget(l, (#l-k))
		end
	end
	setmetatable(log, log_mt)
	
	function addlog(channel, msg, info)
		assert(type(channel) == 'string', "parm#1 Expected string, got "..type(channel))
		assert(type(msg) == 'string', "parm#2 Expected string, got "..type(msg))
		assert((not info) or type(info) == 'table', "parm#2 Expected table, got "..type(info))
		print(string.format('%s: "%s"', channel, msg))
		log[#log+1] = { text = msg, channel= channel or 'system', info = info or {active=false}}
	end
	function addlogf(channel, msg, ...)
		addlog(channel, string.format(msg, ...))
	end
	
	
	local channel = {system=color(.8,.0,.0), talk = color(.0,.0,.8), event = color(.0,.8,.0),
		debug=color(.7,.99,.7)}
	tmp = panel:new{text="",x=50, y=200, width=800-100, height=600-400;
	color_bg = {0,0,0,127}}
	tmp.name="Banana"
	for i=1,(tmp.height/13)+1 do
		local l = panel:new{x=0,y=tmp.height-(i*12)-6, width=tmp.width, height=12; color_bg = {0,0,0,0}}
		l.name = 'logline#'..-i
		l.ondraw = function(self)
			local msg = log[-i + (tmp.offset or 0)]
			self.info = nil
			if msg then
				self.info = msg.info
				local lg = love.graphics
				lg.setColorMode('modulate')
				lg.setColor(channel[msg.channel] or {255, 255, 255})
				lg.setFont(fonts[12])
				lg.print(msg.text, 5, 0)
				lg.setColorMode('replace')
			end
		end
		tmp:add(l)
	end
	addlog('system', "Game Start")
	g:add(tmp)

end

function _M:doturn(...)
	for k, v in pairs(players) do
		v:turn(...)
		if v.hostile then ai_decide(v) end
		if v.active == false then
			self:add(ggfinale:new{x=ui.width*.1, y=ui.height*.1, width=ui.width*.8, height=ui.height*.8,data={players=players}})
		end
	end
	
end

-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M

