local _M = {_NAME = "somepackage", _TYPE = 'module'}

local panel = require "panel"
local icon = require "ggstatusicon"
-------------------------------------------------------------------------
local gg_mt = {__index = _M}
local panel_mt = {__index=panel}
setmetatable(_M, panel_mt)

local ic_blank = {'blank', 'blank', 'blank', 'blank', 'blank', 'blank', 'blank'}

local function ic_update(self, dt)
	local info = self.info
	local eft = self:geteffect()

	if eft then
		info.name = eft.name .." ".. (eft.number or "")
		info.text = eft.text
		info.color = {80,80,80}
		info.active = true
		self.icon = eft.icon
		self.number = eft.number
		self.nonumber = eft.nonumber
	else
		self.nonumber = true
		self.icon = ic_blank
		info.active = false
	end
end

local function ic_geteffect(self)
	return self.parent:geteffect(self.idx)
end

function _M:new(t)
	local ib = panel:new(t)
	setmetatable(ib, gg_mt)
	assert(ib.width >= 25, "Iconbars must be at least 25px wide!")
	assert(ib.height >= 25, "Iconbars must be at least 25px tall!")
	
	ib.player = t.player or {}
	ib.border = false
	ib.color_bg = t.color_bg or {0,0,0,0}
	ib.color_fg = t.color_fg or {255,255,255}
	ib.text = ""

	local rows, cols = ib.height/25, ib.width/25
	local ic
	for row=0,rows-1 do
		for col=0,cols-1 do
			local i = (row*cols)+col+1
			ic = icon:new{x=col*25, y=row*25, width=25, height=25, icons=ic_blank}
			ic.idx = i
			ic.info = {}
			ic.onupdate = ic_update
			ic.geteffect = ic_geteffect

			ib:add(ic)
		end
	end
	
	return ib
end

function _M:geteffect(n)
	return self.player.effect[n]
end

-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M

