local _M = {_NAME = "ggplayerstatus", _TYPE = 'module'}

local panel = require "panel"
local adds = require "adds"
local util = require "panel.util"
local progress = require "panel.progressbar"
local ggstatusicon = require "ggstatusicon"
local ggiconbar = require "ggiconbar"
-------------------------------------------------------------------------

local ggps_mt = {__index = _M}
local panel_mt = {__index=panel}
setmetatable(_M, panel_mt)

function _M:new(t)
	local sp = panel:new(t)
	assert (t.player, "Must be given a valid player reference")
	setmetatable(sp, ggps_mt)
	
	sp.player = t.player
	sp.text = ""
	sp.width = t.width or 200
	sp.height = t.height or 50
	sp.color_bg = t.color_bg or {0,0,0,0}
	sp.color_fg = t.color_fg or {255,255,255}
	sp.border = false
	
	local tmp

	tmp = ggstatusicon:new{x=0, y=0, width=25, height=25, icons={'blue00', 'blue01', 'blue02', 'blue03',  'blue04', 'blue05', 'blue06'}}
	sp.supplies = tmp
	tmp.name = "Supplies"
	tmp.info = {name="Supplies", text = "Stockpiled Items and materials"}
	tmp.text = tostring(sp.player.supplies)
	sp:add(tmp)

	tmp = ggstatusicon:new{x=0, y=25, width=25, height=25, icons={'red00', 'red01', 'red02', 'red03',  'red04', 'red05', 'red06'}}
	sp.ammo = tmp
	tmp.name = "Ammo"
	tmp.info = {name="Ammo", text = "Bullets, munitions, explosive ordancies..."}
	tmp.text = tostring(sp.player.supplies)
	sp:add(tmp)

	tmp = progressbar:new{x=25, y=25, width=175, height=25}
	sp.health = tmp
	tmp.info = {name="Health", text = "Physical health and condition."}
	tmp.name = "health"
	tmp.color_fg = color(.6, .2, .2)
	tmp.color_bg = color(.2, .2, .2)
	tmp.align = 'center' tmp.valign = 'middle'
	sp:add(tmp)

	tmp = ggstatusicon:new{x=25, y=0, width=25, height=25, icons={'deck00', 'deck01', 'deck02', 'deck03',  'deck04', 'deck05', 'deck06'}}
	sp.deck = tmp
	tmp.name = "Deck"
	tmp.info = {name="Deck", text = "The number of cards left in the draw deck"}
	tmp.text = tostring(#sp.player.deck)
	sp:add(tmp)

	tmp = ggiconbar:new{name="iconbar", x=50, y=0, width=sp.width-50, height=25;
	player=sp.player}
	sp.status = tmp
	sp:add(tmp)
	
	return sp
end

function _M:onupdate()
	self.health.progress = self.player.health
	self.health.progressmax = self.player.healthmax or 25
	self.health.progressmin = self.player.healthmin or 0
	self.ammo.number     = self.player.ammo
	self.supplies.number = self.player.supplies
	self.deck.number     = #self.player.deck
end

-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M

