-------------------------------------------------------------------------
-- [button.lua]
-- panel.button
-------------------------------------------------------------------------
-- Copyright (C) Textmode DMB 2011
-------------------------------------------------------------------------
local _M = {_NAME = "button", _TYPE = 'module'}

local panel = require "panel"
-------------------------------------------------------------------------

local btn_mt = {__index=_M}
local panel_mt = {__index=panel}
setmetatable(_M, panel_mt)

function _M:new(...)
	local btn = panel:new(...)
	setmetatable(btn, btn_mt)
	
	btn.name = "button#"..btn.id
	btn.down = false
	return btn
end

function _M:onmousepressed(x, y, btn)
	print(self.name, "mouse down")
	self.down = true
end

function _M:onmousereleased(x, y, btn)
	print(self.name, "mouse up")
	if self.down then
		self:onclick(x, y, btn)
	end
	self.down = false
end

function _M:onmouseexit()
	print(self.name, "mouse exit")
	self.down = false
end

function _M:ondraw()
	local lg = love.graphics
	lg.setColorMode("modulate")
	--print(self.name..".down", self.down)
	
	local fg, bg = self.color_fg, self.color_bg
	local fg2, bg2 = {unpack(fg)}, {unpack(bg)}
	if self.down then
		self.color_fg, self.color_bg = bg2, fg2
		panel.ondraw(self)
	else
--		if self.hovering then
--			fg2[1], fg2[2], fg2[3] = fg2[1]+32, fg2[2]+32, fg2[3]+32
--			bg2[1], bg2[2], bg2[3] = bg2[1]+32, bg2[2]+32, bg2[3]+32
--		end
		self.color_fg, self.color_bg = fg2, bg2
		panel.ondraw(self)
		if self.hovering then
			lg.setColor(255, 255, 255, 50)
			lg.rectangle('fill', 0, 0, self.width, self.height)
		end
	end
	self.color_fg, self.color_bg = fg, bg
end

function _M:onkeypressed(k, u)
	if k == ' ' or k == 'return' or k == 'kpenter' then
		self.down = true
	end
end

function _M:onkeyreleased(k)
	if k == ' ' or k == 'return' or k == 'kpenter' then
		if self.down then
			self:onclick(self.width/2, self.height/2, 'l')
		end
		self.down = false
	end
end

-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M

