-------------------------------------------------------------------------
-- [progressbar.lua]
-- panel.progressbar
-------------------------------------------------------------------------
-- Copyright (C) Textmode DMB 2011
-------------------------------------------------------------------------
local _M = {_NAME = "progressbar", _TYPE = 'module'}

local panel = require "panel"
-------------------------------------------------------------------------

local pb_mt = {__index=_M}
local panel_mt = {__index=panel}
setmetatable(_M, panel_mt)

function _M:new(...)
	local pb = panel:new(...)
	setmetatable(pb, pb_mt)
	
	pb.name = "progressbar#"..pb.id
	pb.progressmax = 100
	pb.progressmin = 0
	pb.progress = 0
	pb.color_bg = {255, 255, 255}
	pb.color_fg = {0, 0, 127}
	pb.font = pb.height * .75
	pb.text = ""
	pb.direction = "left-right" -- only kind implemented atm
	
	return pb
end

function _M:ondraw()
	assert(self.direction == 'left-right', "Only direction=='left-right' is implemented!")

	local lg = love.graphics
	local sis = {lg.getScissor()}
	local cm = lg.getColorMode()
	local absx, absy = self:abspos()
	
	lg.setColorMode('modulate')

	-- begin
	local limit = (self.progressmin-self.progress) / (self.progressmin-self.progressmax)
	limit = limit >= 0 and limit or 0
	
	lg.setFont(fonts[self.font])

	lg.setColor(self.color_bg)
	lg.rectangle('fill',.5,.5,self.width, self.height)

	lg.setColor(self.color_fg)
	lg.rectangle('line',1.5,1.5,self.width-2, self.height-2)
	lg.printf(self.progress, 3,3, self.width-3, 'center')
	lg.rectangle('fill',0,0,self.width*limit, self.height)
	lg.setColor(self.color_bg)
	lg.setScissor(absx, absy, self.width*limit, self.height)	
	lg.printf(self.progress, 3,3, self.width-3, 'center')
	
	lg.setScissor()	
	lg.rectangle('line',1.5,1.5,self.width-2, self.height-2)

	-- restore	
	lg.setColorMode(cm)
	lg.setScissor(unpack(sis))	
end

-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M

