-------------------------------------------------------------------------
-- [textbox.lua]
-- panel.textbox
-------------------------------------------------------------------------
-- Copyright (C) Textmode DMB 2011
-------------------------------------------------------------------------
local _M = {_NAME = "textbox", _TYPE = 'module'}

local panel = require "panel"
-------------------------------------------------------------------------

local tb_mt = {__index=_M}
local panel_mt = {__index=panel}
setmetatable(_M, panel_mt)

function _M:new(...)
	local tb = panel:new(...)
	setmetatable(tb, tb_mt)
	
	tb.name = "textbox#"..tb.id
	tb.color_bg = {0,0,0}
	tb.color_fg = {255, 255, 255}
	tb.font = tb.height * .75
	tb.text = ""
	tb.singleline = true
	tb.filter = ".*"
	return tb
end

local scrub = {kp0='0', kp0='1',kp0='2', kp0='3', kp0='4', kp0='5', kp0='6', kp0='7', kp0='8', kp0='9';
	kpenter='return';}
	
function _M:onkeypressed(k,u)
	print(self.name..' keypressed:', k, u)

	if scrub[k] then k = scrub[k] end

	if k == 'backspace' or k == 'delete' then
		self.text = self.text:sub(1, #self.text-1)
	elseif k == 'return' then
		if  self.singleline then
			self:onclick(self.width/2, self.height/2, 'l')
		else
			self.text = self.text..'\n'
		end
	elseif u and u~=0 then
		-- the next version of love can render unicode, in which case this should work
		-- until then its junk...
		local h, l = math.floor(u % 255), math.floor(u / 255)
		local char = ""
		if l > 0 then char = string.char(l) end
		if h > 0 then char = char..string.char(h) end

		self.text = self.text..self:dofilter(char)
	else
		print(self.name, "no match")
	end
	
end

function _M:dofilter(s)
	if self.filter == nil then
		return s 
	elseif type(self.filter) == 'string' then
		return string.match(s, self.filter)
	elseif type(self.filter) == 'function' then
		return self.filer(s)
	end
	
	error(string.format("Error: invalid textbox filter type; '%s'", type(self.filter)))
	
end


-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M

