local _M = {_NAME = "player", _TYPE = 'module'}

local utils = require "utils"
local deck = require "deck"
local effect = require "effect"
-------------------------------------------------------------------------

local _MT = {__index=_M}

function _M:new(plr)
	local p = {
		name = plr.name or "Survivor";
		healthmax = plr.healthmax or 40;
		health = plr.health or plr.healthmax or 40;
		hostile = not not plr.hostile;
		ammosmin = plr.ammomin or 0;
		ammo = plr.ammo or 3;
		suppliesmin = plr.suppliesmin or 0;
		supplies = plr.supplies or 3;
		hand = {};
		handmax = plr.handmax or 7;
		deck = deck:clonedeck(plr.deck or {});
		effect = {};
		active = true;
	}
	setmetatable(p, _MT)
	assert(p.effect, "What the...?")
	
	if #p.deck == 0 then
		for i=1, 25 do
			p.deck[#p.deck+1] = deck:newcard(pick(deck.set))
		end
	end
	
	p.deck = shuffle(p.deck)
	
	p:drawcard(5)
	
	local e = effect:new(effect.set.plague)
	e.number = 1
	p:addEffect(e)
	
	return p
end


function _M:drawcard(n)
	n = n or 1
--	print("draw:", self.name, n)
	for i=1,n do
		self.hand[#self.hand+1] = self.deck[#self.deck]
		self.deck[#self.deck] = nil
	end
end

local blank_info = {
	name = "";
	text = "";
	color = deck.suit_general;
	active = false
}

local prox_mt = {__index = function(s, k)
	assert(type(s) ~= 'nil' and type(k) ~= 'nil', "WHAT HAS GOD WROUGHT?!")
	local o = rawget(s, 'owner')
	local c = (o and o:card()) or blank_info;
	if o then
		if k == 'text' then
			return rawget(c, k) .. "\n\n[Right-click to discard]"
		else
			return rawget(c, k)
		end
	end
end}



local function hand_ondraw(self)
	local c = self:card()
	if c then
--		print('handdraw', self.player.hostile)
		local c2 = self.player.hostile and {name = "", text ="", color = c.color, image = imgs['generic']} or c
	
		if c.active then
			drawcard(c2, 0, 0, 1)
		end
	end
end

function _M:canafford(card)
	for k, v in pairs(card.cost) do
		if k == 'plague' then
			local p = self:getEffectByType('plague')[1]
			if (not p) or (p.number < v) then
				return false
			end
		else
			assert(self[k], "cannot find "..tostring(k))
			if self[k] < v then
				return false
			end
		end
	end
	return true
end


local function playcard(card, plr)
	assert(type(card) == 'number' and type(plr) == 'table')
	local card, i = plr.hand[card], card
	if not card.active then return end
	
	if not plr:canafford(card) then
		local msg = string.format("%s tried to play '%s', but didn't meet the requirements.", plr.name, card.name)
		addlog('event', msg, card)
		return
	end

	local opp = players[(plr == players[1] and 2) or 1]
	assert(opp ~= plr, "Could not determine opponent")

	plr:applyEffects(card.effect)

	-- correct the numbers...
	for k, v in pairs(players) do
		v.active = v.active and v.health > 0
	end

	table.remove(plr.hand, i)
	local msg = string.format("%s played '%s'.", plr.name, card.name)
	addlog('event', msg, card)

end

function _M:play(card)
	playcard(card, self)
end

function _M:discard(card)
	local card, i = self.hand[card], card
	table.remove(self.hand, i)
	local msg = string.format("%s discarded '%s'.", self.name, card.name)
	addlog('event', msg, card)
end

function _M.modstat(self, stat, amt)
	--check inputs
	assert(type(self) == 'table',  "modstat parm#1: expected table, got "..type(stat))
	assert(type(stat) == 'string', "modstat parm#2: expected string, got "..type(stat))
	assert(type(amt) == 'number', "modstat parm#3: expected number, got "..type(stat))
	assert(self[stat], "No such statistic as '"..stat.."'")
	-- get limits and bounds
	local max = self[stat..'max'] or 1e1000 -- its...big.
	local min = self[stat..'min'] or -1e1000 -- its...-big.
	-- dooooeeeet
	self[stat] = self[stat] + amt 
	-- check limits and bounds
	self[stat] = self[stat] <= max and self[stat] or max
	self[stat] = self[stat] >= min and self[stat] or min
	
end

function _M:turn()
	local v = self
	
	-- draw a card if not distracted
	if #self.hand < self.handmax and not self:getEffectByType('distract')[1] then
		self:drawcard()
	end
	
	if self.health <= 0 then
		self.active = false
	end

	-- do each effect's onturn
	for k, v in pairs(self.effect) do
		print('effects:', self.name, v.name, v.number)
		v:onturn(self)
	end

end

function _M:applyEffects(el)
	assert(type(el) == 'table', "parm#1 expected table (got "..type(el)..")")
	
	local opp = players[(self == players[1] and 2) or 1]
	assert(opp ~= self, "Could not determine opponent")

	for k,v in ipairs(el) do
	
		if effect.set[v[1]] then
			local eff = v[1]
			local num = v[2]
			local prm = v[3]
			if prm then
				prm = { select(3, unpack(prm) )} end
			local e = effect:new(effect.set[eff], prm)
			e.number = num
			self:addEffect(e)
		elseif v[1] == 'opponent' then
			local v = v[2]
			local eff = v[1]
			local num = v[2]
			local prm = v[3]
			if prm then
				prm = { select(3, unpack(prm) )} end
			if effect.set[eff] then
				assert(eff, "Unable to aquire effect name!") 
				assert(effect.set[eff], "Unable to aquire effect: '"..tostring(eff).."'") 
				local e = effect:new(effect.set[eff], prm)
				e.number = num
			else
				e = v
			end
			opp:applyEffects({e})		
		elseif v[1] == 'health' then
			self:modstat('health',v[2])
		elseif v[1] == 'ammo' then
			self:modstat('ammo',v[2])
		elseif v[1] == 'supplies' then
			self:modstat('supplies',v[2])
		elseif v[1] == 'damage' then
			opp:modstat('health',-v[2])
		elseif v[1] == 'enlighten' then
			opp:drawcard()
		elseif v[1] == 'draw' then
			self:drawcard(v[2])
		else
			error("unknown effect: '"..tostring(v[1]).."'")
		end
	end


	-- correct the numbers...
	self.health = self.health < self.healthmax and self.health or self.healthmax
	assert(self.health <= self.healthmax, "player has excessive health")
	self.ammo = self.ammo >= 0 and self.ammo or 0
--		v.plague = v.plague >= 0 and v.plague or 0
	self.supplies = self.supplies >= 0 and self.supplies or 0
	self.active = self.active and self.health > 0

end

function _M:addEffect(e)
	if e.merge then
		local se = self:getEffectByType(e.type)
		for k, v in pairs(se) do
			if v and v.merge then
				v:merge(e)
				return;
			end
		end
	end

	self.effect[#self.effect+1] = e
end

function _M:getEffectByType(t)
	local ret = {}
	for k, v in pairs(self.effect) do
		if v.type == t then
			ret[#ret+1] = v end
	end
	return ret
end

function _M:removeEffect(e)
	if type(e) == number then
		table.remove(self.effect, e)
		return true
	else
		for k, v in pairs(self.effect) do
			if v == e then
				table.remove(self.effect, k)
				return true
			end
		end
	end
	
	return false
end

-- making these named positions would be more readable, but harder to use dynamically...unless we *also* provide a map for number -> named_position.
local position ={{0, 0, (7*75)+25, 100}, {-1, -1, -1, -1}, {0, 600-100, (7*75)+25, 100}, {-1, -1, -1, -1}}

function _M:newhand(t)
	assert(type(t) == 'table' and t.player and t.position)
	
	local pos = position[t.position]
	
	local hand = panel:new{x=pos[1], y=pos[2], width=pos[3], height=pos[4]}
	hand.name = t.name or "Player hand"
	hand.ondraw = function() end

	local tmp
	for i=0,6 do
		tmp = panel:new{x=i*75, y=0, width=100, height=100}
		tmp.name = "Card#"..i+1
		tmp.align = 'center'
		tmp.hostile = not not t.hostile
		function tmp:card() return self.player.hand[i+1], i+1 end
		tmp.ondraw = hand_ondraw
		tmp.player = t.player
		if t.player.hostile then
			tmp.info ={name = "Opponent's Card", text ="A card owned by your Opponent."}
		else
			tmp.info = {owner = tmp}
			setmetatable(tmp.info, prox_mt)
			function tmp.onclick(s, x, y, btn)
				if s:card() then
					local _, c = s:card();
					if btn == 'l' then
						s.player:play(c)
					elseif btn == 'r' then
						s.player:discard(c)
					end
				end
			end
			function tmp.hover(s, ...)
				local c = s:card()
				if not (c and c.active) then 
					return false
				else
					return panel.hover(s, ...)
				end
			end
		end
		hand:add(tmp)
	end

	return hand
end

-------------------------------------------------------------------------
if _VERSION == "Lua 5.1" then _G[_M._NAME] = _M end

return _M

